//	Atom Hard Disk interface
//
//	NOTES:
//		Most of this is still a disgusting hack from experimenting with BDOS and the ATA(PI) specs.
//
//		It used to work quite well with earlier versions of BDOS, before CD-ROM support was added.
//		I've since added in some parts of the ATAPI detection to see what it can do, but it doesn't
//		go as far as processing specific packet commands to do any real work.  Even without the ATAPI
//		stuff it doesn't do enough for it to work with recent BDOS versions, tho it may still
//		work with older versions if a (currently hard-coded) empty 4MB HARDDISK.DAT file is created
//		for it.
//
//		To be looked at soon hopefully!


#include "stdafx.h"

#include "Atom.h"
#include "Util.h"

namespace Atom
{

BYTE abDiskSectorData[512];

BYTE*	m_pbBuffer;
UINT	m_uBuffer;


typedef struct tagATAregs
{
	WORD	wData;			// 0x1f0
	WORD	wError;			// 0x1f1, init=1

	WORD	wSectorCount;	// 0x1f2, init=1
	WORD	wSector;		// 0x1f3, init=1

	WORD	wCylinderLow;	// 0x1f4, init=0
	WORD	wCylinderHigh;	// 0x1f5, init=0

	WORD	wDriveHead;		// 0x1f6, init=0

	WORD	wStatus;		// 0x1f7 (read-only)
	WORD	wCommand;		// 0x1f7 (write-only)

	WORD	wDriveAddress;	// 0x3f7
}
ATAregs;

ATAregs m_sRegs;


// Read registers

FILE*	hHardDisk;


DRIVEIDENTITY sDriveIdentity = 
{
	0x7146,						// Bit mapped general configuration information. True when bit set
	15,							// Number of logical cylinders in the default translation mode
	0,							// Reserved
	16,							// Number of logical heads in the default translation mode
	63*512,						// Number of unformatted bytes per logical track
	512,						// Number of unformatted bytes per sector
	63,							// Number of logical sectors per track
	0,							// Bits 15...08: Inter Sector Gap after Index & before splice
								// Bits 07...00: Inter Sector Gap bytes
	0,							// Bits 15...08: Reserved
								// Bits 07...00: Bytes in Phase Lock Oscillator field
	0,							// Number of vendor unique status words

	"        0 214365879",		// Serial number, 20 ASCII chars, right aligned & padded with spaces

	0,							// Controller Type:
								// 0000h: Not specified
								// 0001h: Single ported, single sector buffer capable of data transfers only
								//			to or from the host or the disk at one time
								// 0002h: Dual ported, multiple sector buffer capable of simultaneous data
								//			transfers to and from the host, or from the host and the disk
								// 0003h: Dual ported, multiple sector buffer capable of simultaneous data
								//			transfers with read caching
								// 0004h-FFFFh: Reserved
	1024/512,					// Buffer size in 512 byte increments
	0,							// Number of ECC bytes passed to host on R/W long operations
	".008   ",					// Firmware revision, 8 ASCII chars, left aligned & space padded
	"iSDmsi k                               ",	// Model Number, 40 ASCII chars, left aligned & space padded

	0,							// READ/WRITE multiples implemented
	0,							// Supports double word I/O transfer

	0,							// Reserved
	0,							// Reserved

	1,							// Minimum PIO data transfer cycle time in nsec
	1,							// Minimum DMA data transfer cycle time in nsec
};


bool Init ()
{
	hHardDisk = fopen("HARDDISK.DAT", "r+b");
	return true;
}

void Exit ()
{
	if (hHardDisk)
	{
		fclose(hHardDisk);
		hHardDisk = NULL;
	}
}

BYTE In (WORD wPort_)
{
	switch (wPort_ & 0xff)
	{
		case 0xf5:
			TRACE("Disk read from 0xf5 ???\n");
			DebugBreak();
			break;

		case 0xf6:
		case 0xf7:
		{
			bool fHigh = ((wPort_ & 0xff) == 0xf6);	// TRUE if we're reading the high byte

			BYTE bRet = 0xff;

			switch (m_sRegs.wCommand)
			{
				case 0xdf:
					TRACE("Disk: READ *RESET*\n");
					break;

				case 0xef:
					TRACE("Disk: READ drive address register\n");
					bRet = fHigh ? HIBYTE(m_sRegs.wDriveAddress) : LOBYTE(m_sRegs.wDriveAddress);
					break;

				case 0xf0:
					// Return zero if no more data is available
					if (!m_uBuffer)
					{
						TRACE("Returning zero (no more data)\n");
						return 0;
					}
					else if (m_uBuffer == sizeof abDiskSectorData)
						TRACE("Disk: READ data wPort_ (first of %d bytes)\n", m_uBuffer);


					// 1 less byte available to read
					m_uBuffer--;

					if (fHigh)
						return m_pbBuffer[1];

					m_pbBuffer += 2;
//					TRACE("Returning LOW byte: %#02x (%d), %d byte(s) remaining\n", m_pbBuffer[-2], pbDiskData[-2], m_uBuffer);
					return m_pbBuffer[-2];
					break;

				case 0xf1:
					TRACE("Disk: READ error register\n");
					break;

				case 0xf2:
					TRACE("Disk: READ sector count\n");
					break;

				case 0xf3:
					TRACE("Disk: READ sector number\n");
					break;

				case 0xf4:
					TRACE("Disk: READ cylinder low\n");
					break;

				case 0xf5:
					TRACE("Disk: READ cylinder high\n");
					return 0x20;
					break;

				case 0xf6:
					TRACE("Disk: READ drive/head\n");
//						return 0xa0;
					break;

				case 0xf7:
					TRACE("Disk: READ status\n");
					bRet = fHigh ? HIBYTE(m_sRegs.wStatus) : LOBYTE(m_sRegs.wStatus);
					break;

				default:
					TRACE("Read from %#04x (%d)\n", (int)wPort_, (int)(wPort_&0xff));
//					TRACE("Disk: READ <unknown> %#04x (%d)\n", wPort_, wPort_&0xff);
					break;
			}

			TRACE("Returning %#02x (%d)\n", bRet, bRet);
			return bRet;
		}
		break;

		default:
			TRACE("Read from %#04x (%d)\n", (int)wPort_, (int)(wPort_&0xff));
//			TRACE("*** Disk: READ <wPort_ unknown> %#04x (%d)\n", wPort_, wPort_&0xff);
	}

	return 0xff;
}


void Out (WORD wPort_, BYTE bVal_)
{
	switch (wPort_ & 0xff)
	{
		case 0xf5:
		{
			m_pbBuffer = NULL;
			m_uBuffer = 0;

			switch (m_sRegs.wCommand = bVal_)
			{
				case 0xef:
//					TRACE("Disk write: selected drive address register\n");
					m_sRegs.wStatus |= STATUS_DRQ;
					break;

				case 0xf0:
//					TRACE("Disk write: selected DATA PORT\n");
					m_pbBuffer = abDiskSectorData;
					m_uBuffer = sizeof abDiskSectorData;

					// Ready to read or write data
					m_sRegs.wStatus |= STATUS_DRQ;
					break;

				case 0xf1:	/*TRACE("Disk write: selected ERROR\n");			*/break;
				case 0xf2:	/*TRACE("Disk write: selected SECTOR COUNT\n");		*/break;
				case 0xf3:	/*TRACE("Disk write: selected SECTOR NUMBER\n");	*/break;
				case 0xf4:	/*TRACE("Disk write: selected CYLINDER LOW\n");		*/break;
				case 0xf5:	/*TRACE("Disk write: selected CYLINDER HIGH\n");	*/break;
				case 0xf6:	/*TRACE("Disk write: selected DRIVE/HEAD\n");		*/break;

				case 0xf7:
//					TRACE("Disk write: selected COMMAND/STATUS\n");
//					wStatus |= STATUS_DRQ;
					break;

				case 0xf8:
//					TRACE("Disk write: selected STATUS\n");
					break;

				default:
					TRACE("Write to %#04x (%d), %#02x (%d)\n", (int)wPort_, (int)(wPort_&0xff), (int)bVal_, (int)bVal_);
			}
		}
		break;

		case 0xf6:
		case 0xf7:
		{
			WORD* pwReg = NULL;
			bool fHigh = ((wPort_ & 0xff) == 0xf6);	// TRUE if we're writing the high byte

			switch (m_sRegs.wCommand)
			{
				case 0xf0:
					// Data expected?
					if (m_uBuffer)
					{
						// 1 less byte expected
						m_uBuffer--;

						// Write the data in BIG endian order
						if (fHigh)
							m_pbBuffer[1] = bVal_;
						else
						{
							m_pbBuffer[0] = bVal_;
							m_pbBuffer += 2;
						}

						// Received eveything we need?
						if (!m_uBuffer)
						{
							switch (m_sRegs.wCommand)
							{
								case 0xf0:
								{
									// Write the sector to disk if it's full
									long lWritten = fwrite(abDiskSectorData, 1, sizeof abDiskSectorData, hHardDisk);
								}

								default:
									TRACE("Received all data\n");
							}
						}
/*
						// Return zero if no more data is available
						if (m_pbBuffer >= (abDiskSectorData + sizeof abDiskSectorData))
						{
							TRACE("Disk write: no more space available!\n");
							return;
						}
						else if (m_pbBuffer == abDiskSectorData && !fHigh)
							TRACE("Disk: WRITE data wPort_ (first of %d bytes)\n", m_uBuffer);
*/
					}
					return;

				case 0xf1:
					TRACE("Disk: WRITE PRECOMPENSATION\n");
					break;

				case 0xf2:
					TRACE("Disk: WRITE sector count = %d\n", bVal_);
					pwReg = &m_sRegs.wSectorCount;
					break;

				case 0xf3:
					TRACE("Disk: WRITE sector number = %d\n", bVal_);
					pwReg = &m_sRegs.wSector;
					break;

				case 0xf4:
					TRACE("Disk: WRITE cylinder low = %d\n", bVal_);
					pwReg = &m_sRegs.wCylinderLow;
					break;

				case 0xf5:
					TRACE("Disk: WRITE cylinder high = %d\n", bVal_);
					pwReg = &m_sRegs.wCylinderHigh;
					break;

				case 0xf6:
				{
					TRACE("Disk: WRITE drive/head = %d\n", bVal_);
					pwReg = &m_sRegs.wDriveHead;

					if (!fHigh)
					{
						// Set the relevant drive active bit in bit 0 or 1
						m_sRegs.wDriveAddress = (!(bVal_ & 0x10)) ? 0x0001 : 0x0002;

						// Store the head selected in bits 2 to 5
						m_sRegs.wDriveAddress |= (bVal_ & 0xf) << 2;

						// Drive ready and selected head settled
						m_sRegs.wStatus |= (STATUS_DRDY|STATUS_DSC);
					}
				}
				break;

				case 0xf7:
					switch (bVal_)
					{
						case 0xec:
							TRACE("Disk command: IDENTIFY\n");
							memset(&abDiskSectorData, 0, sizeof abDiskSectorData);
							memcpy(&abDiskSectorData, &sDriveIdentity, sizeof sDriveIdentity);
							m_sRegs.wStatus = STATUS_DRQ;
							break;
/*
						case 0x08:
							TRACE("Disk command: ATAPI Soft Reset\n");

							m_sRegs.wError = 0x01;
							m_sRegs.wSectorCount = m_sRegs.wSector = 0x01;
							m_sRegs.wDriveHead = 0x00;

							// ATAPI signature
							m_sRegs.wCylinderLow = 0xeb;
							m_sRegs.wCylinderHigh = 0x14;

							m_sRegs.wStatus = 0;
							break;
*/
						case 0x20:
						{
							TRACE("Disk command: Read Sectors With Retry\n");
							memset(&abDiskSectorData, 0, sizeof abDiskSectorData);
							m_sRegs.wStatus = STATUS_DRQ;

							DWORD dwCyclinders = (((DWORD)m_sRegs.wCylinderHigh<<16)|m_sRegs.wCylinderLow);

							long lPos = (m_sRegs.wSector +
									((m_sRegs.wDriveAddress >> 2) & 0x0f) * sDriveIdentity.wSectorsPerTrack +
									dwCyclinders * (sDriveIdentity.wLogicalHeads * sDriveIdentity.wSectorsPerTrack))
									* sDriveIdentity.wBytesPerSector;
							TRACE("Reading CHS: %d:%d:%d (%#08lx)\n", dwCyclinders, (m_sRegs.wDriveAddress >> 2) & 0x0f, m_sRegs.wSector, lPos);
							fseek(hHardDisk, lPos, SEEK_SET);

							long lRead = fread(&abDiskSectorData, 1, sDriveIdentity.wBytesPerSector, hHardDisk);

							// Set the sector buffer pointer and how much we have available to read
							m_pbBuffer = abDiskSectorData;
							m_uBuffer = sizeof abDiskSectorData;
						}
						break;

						case 0x21:
							TRACE("Disk command: Read Sectors\n");
							memset(&abDiskSectorData, 0, sizeof abDiskSectorData);
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x22:
						case 0x23:
							TRACE("Disk command: Read Long (with or without retry)\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;
							
						case 0x30:
						{
							TRACE("Disk command: Write Sectors With Retry\n");
							memset(&abDiskSectorData, 0, sizeof abDiskSectorData);
							m_sRegs.wStatus = STATUS_DRQ;

							DWORD dwCylinders = (((DWORD)m_sRegs.wCylinderHigh<<16)|m_sRegs.wCylinderLow);

							long lPos = (m_sRegs.wSector +
									((m_sRegs.wDriveAddress >> 2) & 0x0f) * sDriveIdentity.wSectorsPerTrack +
									dwCylinders * (sDriveIdentity.wLogicalHeads * sDriveIdentity.wSectorsPerTrack))
									* sDriveIdentity.wBytesPerSector;
							TRACE("Writing CHS: %d:%d:%d (%#08lx)\n", dwCylinders, (m_sRegs.wDriveAddress >> 2) & 0x0f, m_sRegs.wSector, lPos);
							fseek(hHardDisk, lPos, SEEK_SET);

							long lRead = fread(&abDiskSectorData, 1, sDriveIdentity.wBytesPerSector, hHardDisk);
							fseek(hHardDisk, lPos, SEEK_SET);

							// Set the sector buffer pointer and how much space we have available for writing
							m_pbBuffer = abDiskSectorData;
							m_uBuffer = sizeof abDiskSectorData;
						}
						break;

						case 0x31:
							TRACE("Disk command: Write Sectors\n");
							memset(&abDiskSectorData, 0, sizeof abDiskSectorData);
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x32:
						case 0x33:
							TRACE("Disk command: Write Long (with or without retry)\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x40:
						case 0x41:
							TRACE("Disk command: Verify Sector (with or without retry)\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x50:
							TRACE("Disk command: Format Track\n");
							TRACE("Formatting track %d\n", m_sRegs.wSectorCount);

							// Format data required from host
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x90:
							TRACE("Disk command: Execute Drive Diagnostic\n");
							m_sRegs.wError = 0;
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x91:
							TRACE("Disk command: Initialize Drive Parameters \n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;
							
						case 0x94:
						case 0xe0:
							TRACE("Disk command: Standby Immediate\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x95:
						case 0xe1:
							TRACE("Disk command: Idle Immediate\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0x96:
						case 0xe2:
							TRACE("Disk command: Standby\n");	// DEVICE OFF
							m_sRegs.wStatus = STATUS_DRQ;
							break;
/*
						case 0xa0:
							TRACE("Disk command: ATAPI Packet Command\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;
*/
						case 0xe4:
							TRACE("Disk command: Read Buffer\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;

						case 0xef:
							TRACE("Disk write: Set features\n");
							m_sRegs.wStatus = STATUS_DRQ;
							break;
							
						default:
							switch (bVal_ & 0xf0)
							{
								case 0x10:
									TRACE("Disk command: Recalibrate\n");
									m_sRegs.wStatus = STATUS_DRQ;
									break;

								case 0x70:
									TRACE("Disk command: Seek\n");
									m_sRegs.wStatus = STATUS_DRQ;
									break;

								default:
									TRACE("Write to %#04x (%d), %#02x (%d)\n", (int)wPort_, (int)(wPort_&0xff), (int)bVal_, (int)bVal_);
									m_sRegs.wError = ERROR_ABRT;
									m_sRegs.wStatus = STATUS_ERROR;
							}
							break;
					}
					break;

				default:
					TRACE("Write to %#04x (%d), %#02x (%d)\n", (int)wPort_, (int)(wPort_&0xff), (int)bVal_, (int)bVal_);
					break;
			}

			if (pwReg)
			{
				if (fHigh)
					*pwReg = (((WORD)bVal_) << 8) | LOBYTE(*pwReg);
				else
					*pwReg = HIBYTE(*pwReg) | (WORD)bVal_;
			}
			break;
		}

		default:
			TRACE("Write to %#04x (%d), %#02x (%d)\n", (int)wPort_, (int)(wPort_&0xff), (int)bVal_, (int)bVal_);
	}
}

};	// namespace Atom
