//	NOTES:
//		This should probably go in FDC.cpp, or that stuff moved into here.

#include "stdafx.h"

#include "CPU.h"
#include "CDrive.h"

////////////////////////////////////////////////////////////////////////////////

// Time motor stays on after no further activity:  10 revolutions at 300rpm = 2 seconds
#define FLOPPY_MOTOR_ACTIVE_TIME		(2 * EMULATED_TSTATES_PER_SECOND)


void CDrive::ResetAll ()
{
	// No disk inserted
	m_pDisk = NULL;	

	// Track 0, sector 1 and head over track 0
	memset(&m_sRegs, 0, sizeof m_sRegs);
	m_sRegs.bSector = 1;
	m_sRegs.bData = 0xff;
	m_nHeadPos = 0;

	// No data available for reading
	m_pbBuffer = NULL;
	m_uBuffer = 0;
}


// Insert a new disk from the named source (usually a file)
bool CDrive::Insert (const char* pcszSource_)
{
	CDisk* pDisk = CDisk::Open(pcszSource_);
	if (pDisk)
		Insert(pDisk);

	return pDisk != NULL;
}

// Insert a new disk object
void CDrive::Insert (CDisk* pDisk_)
{
	// Make sure we eject any disk that's already there
	Eject();

	// Save the disk pointer for use later
	m_pDisk = pDisk_;
}

// Eject any inserted disk
CDisk* CDrive::Eject (bool fDelete/*=true*/)
{
	CDisk* pDisk = m_pDisk;
	m_pDisk = NULL;

	// Disk the disk object if required
	if (pDisk && fDelete)
	{
		delete pDisk;
		pDisk = NULL;
	}

	// Return a pointer to the disk, or NULL if we deleted it
	return pDisk;
}

bool CDrive::Save ()
{
	return (m_pDisk && (m_pDisk->IsModified() || m_pDisk->Save()));
}

void CDrive::FrameEnd ()
{
	// If the motor hasn't been used for 2 seconds, switch it off
	if (IsMotorOn() && ((g_dwCycleCounter - m_dwLastMotorOn) >= FLOPPY_MOTOR_ACTIVE_TIME))
		m_sRegs.bStatus &= ~MOTOR_ON;
}

////////////////////////////////////////////////////////////////////////////////
