#ifndef OPTION_H
#define OPTION_H

namespace Options
{
	bool Load (int argc_, char* argv[]);
	void Save ();


typedef struct
{
	char logfile[256];		// log filename

	int fastreset;			// non-zero for a fast system reset
	int autoboot;			// non-zero to press F9 on start
	int spectrum;			// start in Spectrum mode
	int boundaries;			// non-zero for more accurate boundary timings

	int video;				// Non-zero if video is enabled
	int sync;				// Attempt to syncronise the emulator to 50Hz
	int frameskip;			// 1 to 4 to skip every 0 to 3 SAM frames
	int sourcesize;			// 1x1 or 2x1 source image
	int scale;				// Window scaling mode
	int ratio5_4;			// Use 5:4 screen ratio
	int fullscreen;			// non-zero to start in full-screen mode
	int depth;				// screen depth for full-screen
	int profile;			// show profile stats
	int floppyleds;			// show floppy drive LEDs
	int videomethod;		// method to use to display video
	int drawmethod;			// method to use to draw SAM screen image

	char rom0[_MAX_PATH];	// SAM ROM 0 image filename
	char rom1[_MAX_PATH];	// SAM ROM 1 image filename
	char spectrumrom[_MAX_PATH]; // Default Spectrum ROM
	int mainmem;			// 256 or 512 for amount of main memory
	int externalmem;		// No. Mb of external memory

	char disk1[_MAX_PATH];	// floppy disk image in drive 1
	char disk2[_MAX_PATH];	// floppy disk image in drive 2
	int drive1;				// non zero if drive 1 is present
	int drive2;				// non zero if drive 2 is present

	int keyboard;			// Keyboard mode (raw/SAM/Spectrum)
	int altforcntrl;		// Non-zero if Left-Alt is used for SAM Cntrl
	int mouse;				// non-zero if we want the SAM mouse
	int joystick1;			// joystick 1 device number
	int joystick2;			// joystick 2 device number
	int joysticktol;		// joystick displacement tolerance

	int parallel1;			// parallel port 1
	int parallel2;			// parallel port 2

	int sambus_clock;		// non zero if we want SAMBUS clock support
	int dallas_clock;		// non zero if we want DALLAS clock support

	int sound;				// non-zero if we want sound enabled
	int rate;
	int bits;
	int stereo;
	int filter;
	int	latency;
	bool dacoptimise;
}
OPTIONS;

extern OPTIONS g_sOptions;

};	// namespace Options


#define GetOption(field)	Options::g_sOptions.field
#define SetOption(field,value)	SetOption_(Options::g_sOptions.field, value)

inline void SetOption_(bool& rfOption_, bool fValue_)		{ rfOption_ = fValue_; }
inline void SetOption_(int& rnOption_, int nValue_)			{ rnOption_ = nValue_; }
inline void SetOption_(char* pszOption_, char* pszValue_)	{ strcpy(pszOption_, pszValue_); }


#endif	// OPTION_H
