//	Notes:
//		Still needs generic implementation for high-resolution timing values.

#include "stdafx.h"

#include "Options.h"
#include "Profile.h"


/*
BeOS: system_time  ??

Unix:

	struct timespec t;
	clock_gettime(CLOCK_REALTIME, &t);
	uint64 tl = (uint64)t.tv_sec * 1000000 + t.tv_nsec / 1000;

OR
	struct timeval t;
	gettimeofday(&t, NULL);
	uint64 tl = (uint64)t.tv_sec * 1000000 + t.tv_usec;

THEN
	hi = tl >> 32;
	lo = tl;
*/

////////////////////////////////////////////////////////////////////////////////

namespace Profile
{

PROFILE g_sProfile;

static __int64 llTotal, llLast;

__int64* allStack[10];
UINT uStackPos = 0;

void Reset ()
{
	llTotal = 0;
	memset(&g_sProfile, 0, sizeof g_sProfile);
	memset(&allStack, 0, sizeof allStack);
	uStackPos = 0;
#ifdef _WIN32
	QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER*>(&llLast));
#endif
}

void ProfileUpdate ()
{
	__int64 llNow, llElapsed;

	// Work out how much time has passed since the last check
#ifdef _WIN32
	QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER*>(&llNow));
	llElapsed = llNow - llLast;
#else
	llNow = llElapsed = 0;
#endif

	// If there was an item set, update it with the elapsed time
	if (allStack[uStackPos])
	{
		*allStack[uStackPos] += llElapsed;
		llTotal += llElapsed;
	}

	llLast = llNow;
}

void ProfileStart_(__int64* pllNew_)
{
	if (GetOption(profile))
	{
		ProfileUpdate();

		// Remember the new item to time and the start time
		if (uStackPos < (sizeof allStack / sizeof allStack[0]))
			allStack[++uStackPos] = pllNew_;
	}
}

void ProfileEnd ()
{
	if (GetOption(profile))
	{
		ProfileUpdate();

		if (uStackPos)
			uStackPos--;
	}
}


#define AddStat(x)		sprintf(sz + strlen(sz), "  %s:%d%%", #x, static_cast<int>((g_sProfile.ll##x * 100UL) / llTotal))

const char* GetStats ()
{
	static char sz[64];

	sz[0] = '\0';

	if (llTotal)
	{
		AddStat(CPU);
		AddStat(GFX);
		AddStat(Sound);
		AddStat(Blit);
		AddStat(Idle);
	}

	return sz;
}

};	// namespace Profile
