#ifndef SAM_H
#define SAM_H


// Constants defining screen height and the visible portion of it
#define HEIGHT_LINES				312											// Total generated screen lines
#define SCREEN_LINES				192											// Lines in main screen area
#define TOP_BORDER_LINES			68											// Lines above main screen area
#define BOTTOM_BORDER_LINES			(HEIGHT_LINES-SCREEN_LINES-TOP_BORDER_LINES)
#define VISIBLE_LINES				/*36*/24									// Lines visible above main screen area
#define OFFSCREEN_LINES				(TOP_BORDER_LINES-VISIBLE_LINES)

// Constants defining screen width and the visible portion of it (in 8 'pixel' blocks)
#define WIDTH_BLOCKS				48											// Total generated screen width
#define SCREEN_BLOCKS				32											// Blocks in main screen area
#define BORDER_BLOCKS				((WIDTH_BLOCKS-SCREEN_BLOCKS) >> 1)			// Total blocks for left and right border
#define VISIBLE_BLOCKS				2											// Blocks of each border normally visible
#define OFFSCREEN_BLOCKS			(BORDER_BLOCKS-VISIBLE_BLOCKS)
#define BORDER_PIXELS				(BORDER_BLOCKS << 3)
#define SCREEN_PIXELS				(SCREEN_BLOCKS << 3)


// Constants for video/frame timing
#define TSTATES_PER_LINE			(WIDTH_BLOCKS << 3)
#define TSTATES_PER_FRAME			(TSTATES_PER_LINE * HEIGHT_LINES)
#define REAL_TSTATES_PER_SECOND		6000000UL										// SAM's Z80b CPU runs at 6MHz
#define REAL_FRAMES_PER_SECOND		(REAL_TSTATES_PER_SECOND / TSTATES_PER_FRAME)	// Actually 50.08
#define EMULATED_FRAMES_PER_SECOND	50
#define EMULATED_TSTATES_PER_SECOND	(EMULATED_FRAMES_PER_SECOND * TSTATES_PER_FRAME)

#define USECONDS_TO_TSTATES(x)		(static_cast<DWORD>(x) * (REAL_TSTATES_PER_SECOND / 1000000))

#define VIDEO_DELAY					8		// t-states by which contention precedes the actual screen as seen


#define MEM_PAGE_SIZE				0x4000	// Memory pages are 16K
#define N_PAGES_MAIN				32		// Number of pages in the SAM's main memory (512K)
#define N_PAGES_1MB					64		// Number of pages in 1MB of memory
#define MAX_EXTERNAL_MB				4		// SAM supports 0 to 4 MB of external memory


#define N_PALETTE_COLOURS			128		// 128 colours in the SAM palette
#define N_CLUT_REGS					16		// 16 CLUT entries
#define N_SAA_REGS					32		// 32 registers in the Philips SAA1099 sound chip


#endif	// SAM_H
