#include "stdafx.h"

#include "Frame.h"
#include "FrameBuf.h"
#include "CPU.h"
#include "Util.h"
#include "Options.h"
#include "Video.h"
#include "CDrive.h"
#include "Profile.h"

extern CDisplay* g_pDisplay;
extern DWORD g_dwCycleCounter;

// SAM palette colours to use for the floppy drive LED on/off
#define LED_OFF_COLOUR			8
#define LED_ON_COLOUR			70


namespace Frame
{
static int nLastLineNo, nLastVideoChangePos;

static int nFrame, nDrawnFrames;	// Frame count in last second and number of those actually drawn
static bool fDrawFrame = true;
static DWORD dwLastTime;

static char g_szStatus[128], g_szProfile[128];

static BYTE* pbFrameMem;

/*
// Timer handler, called every 20ms
void CALLBACK TimeCallback (UINT uTimerID_, UINT uMsg_, DWORD dwUser_, DWORD dw1_, DWORD dw2_)
{
	// Signal that the next frame is due
	SetEvent(g_hEvent);

	// We could do with updating the input in here to give a better keyboard response
	// for slow machines, but DirectInput does nasty things if we try!
}
*/

bool Init ()
{
	// Clear the frame buffer surfaces
	Clear();

	pbFrameMem = new BYTE[10000000];
	return true;
}

void Exit ()
{
	if (pbFrameMem)
	{
		delete pbFrameMem;
		pbFrameMem = NULL;
	}
}


void Start ()
{
	nLastLineNo = nLastVideoChangePos = 0;
}

void Update (int nLine_, int nLineCycle_)
{
	if (!fDrawFrame)
		return;

	ProfileStart(GFX);

	// If we're still on the same line as last time we just need to update part of a line
	if (nLine_ == nLastLineNo)
	{
		int nPosition = nLineCycle_ >> 3;
//		g_pDisplay->UpdateLine(nLine_, nLastVideoChangePos, nPosition);
		nLastVideoChangePos = nPosition+1;
	}

	// We've got a block to update
	else
	{
//		TRACE("%d:%d to %d:%d\n", nLastLineNo, nLastVideoChangePos, LineNo, nLineCycle_ >> 3);

		// Finish the rest of the line we updated
//		g_pDisplay->UpdateLine(nLastLineNo, nLastVideoChangePos, TSTATES_PER_LINE >> 3);

		// Update any complete lines in the middle
//		for (int nLine = nLastLineNo+1 ; nLine < nLine_ ; nLine++)
//			g_pDisplay->UpdateLine(nLine, 0, TSTATES_PER_LINE >> 3);

		// Update to where we've reached on the current line
		int nPosition = nLineCycle_ >> 3;
//		g_pDisplay->UpdateLine(nLine_, 0, nPosition);

		// Remember where we've drawn up to so we can continue next time
		nLastLineNo = nLine_;
		nLastVideoChangePos = nPosition+1;
	}

	ProfileEnd();
}

void End ()
{
	// If the current frame is being draw, update the image and counter
	if (fDrawFrame)
	{
		Update(HEIGHT_LINES, WIDTH_BLOCKS);
		nDrawnFrames++;
	}

	nFrame++;

	// Whether the next frame gets draw depends on the frame-skip setting
	fDrawFrame = GetOption(video) && (!nDrawnFrames || (GetOption(frameskip) ? !(nFrame % GetOption(frameskip)) :
					1));//WaitForSingleObject(g_hEvent, 0) != WAIT_OBJECT_0));

	// Frame synchronisation required?
	ProfileStart(Idle);
	switch (GetOption(sync))
	{
		// Wait for timer to trigger the event
		case 1:
//			if (g_hEvent)
//				WaitForSingleObject(g_hEvent, INFINITE);
			break;

		// 2 vsync, useful for a smooth 50fps when using a display mode with 100Hz vertical refresh
		case 3:
			Video::WaitVBlank();
			// Fall through...

		// 1 vsync, not so useful, but will give a smooth 60fps for a display mode with 60Hz vertical refresh
		case 2:
			Video::WaitVBlank();
			break;

		// No sync
		default:
			break;
	}
	ProfileEnd();

//	ResetEvent(g_hEvent);
}

bool Lock (BYTE** pFrame_, LONG* plPitch_)
{
	*pFrame_ = pbFrameMem;
	*plPitch_ = 0;

	return true;
}

void Unlock ()
{
}

void Clear ()
{
}

void Redraw ()
{
	if (!fDrawFrame)
		return;

	// Source image size
//	RECT rectFrom = { 0,0, (HSize1xX * GetOption(sourcesize)), VISIBLE_LINES + SCREEN_LINES + VISIBLE_LINES }, rectTo;

	int nXmargin = ((OFFSCREEN_BLOCKS << 3) - BorderWidth_H_1xX) * GetOption(sourcesize);
	int nYmargin = ((VISIBLE_LINES + SCREEN_LINES + VISIBLE_LINES) - VSizeXx1) / 2;


	// Full screen mode uses the full target, resulting in more black border than the window
	if (GetOption(fullscreen))
	{
	}

	// Windowed output only displays the visible part
	else
	{
	}

	// If enabled, draw the floppy LEDs in the bottom-left of the display
	if (GetOption(floppyleds))
	{
	}

	ProfileStart(Blit);

	ProfileEnd();
}


}	// namespace Frame
