#include "stdafx.h"

#include "Z80.h"
#include "Options.h"
#include "Util.h"
#include "Mouse.h"
#include "Floppy.h"
#include "Memory.h"
#include "Sound.h"
#include "Input.h"
#include "Video.h"
#include "Frame.h"
#include "CDrive.h"
#include "Z80.h"
#include "Version.h"

extern Z80Regs regs;

extern CDrive g_Drive1, g_Drive2;

#include "FrameBuf.h"
extern CDisplay* g_pDisplay;
#include "resource.h"	// For menu and dialogue box symbols

BOOL CALLBACK ImportExportDlgProc (HWND hdlg_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_);
void CentreWindow (HWND hwnd_, HWND hwndParent_=NULL);
bool init_display_depth ();

bool g_fActive = true;

CDisplay* g_pDisplay;



bool InitWindow ();

namespace UI
{

bool Init ()
{
	return true;
}

void Exit ()
{
}


// Check and process any incoming messages
bool CheckEvents ()
{
	return true;
}

};


void InsertDisk (CDrive& rDrive_)
{
	char szFile[_MAX_PATH] = "";

	CDisk* pDisk = rDrive_.Eject(false);
	if (pDisk)
		strcpy(szFile, pDisk->GetName());

	bool fReadOnly = pDisk ? pDisk->IsReadOnly() : false;
	if (GetLoadFile(hwnd, szDiskFilters, NULL, szFile, sizeof szFile, &fReadOnly))
	{
		CDisk *pNewDisk = CDisk::Open(szFile, fReadOnly);
		if (!pNewDisk)
			Message(msgWarning, "Invalid disk image:\n\n%s", szFile);
		else
		{
			delete pDisk;
			pDisk = pNewDisk;
		}
	}

	if (pDisk)
		rDrive_.Insert(pDisk);
}


void ResizeWindow ()
{
/*
	int nWidth = (VISIBLE_BLOCKS + SCREEN_BLOCKS + VISIBLE_BLOCKS) << 3;
	int nHeight = VISIBLE_LINES + SCREEN_LINES + VISIBLE_LINES;

	if (GetOption(ratio5_4))
		nWidth = MulDiv(nWidth, 5, 4);

	if (GetOption(fullscreen))
	{
		DWORD dwStyle = GetWindowStyle(hwnd);
		dwStyle = WS_POPUP|WS_VISIBLE;
		SetWindowLong(hwnd, GWL_STYLE, dwStyle);

		RECT rect = { 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN) };
		SetWindowPos(hwnd, HWND_TOPMOST, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, 0);
	}
	else
	{
		DWORD dwStyle = (GetWindowStyle(hwnd) & WS_VISIBLE) | WS_OVERLAPPEDWINDOW;
		SetWindowLong(hwnd, GWL_STYLE, dwStyle);

		RECT rect = { 0, 0, (nWidth * GetOption(scale)), (nHeight * GetOption(scale)) };

		// Adjust the window rectangle to give the client area size required for the main window
		AdjustWindowRectEx(&rect, GetWindowStyle(hwnd), GetMenu(hwnd) != NULL, GetWindowExStyle(hwnd));
		SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_NOMOVE);

		// Get the actual client rectangle, now that the menu is in place
		RECT rClient;
		GetClientRect(hwnd, &rClient);

		// If the menu has wrapped the client area won't be big enough, so enlarge if necessary
		if ((rClient.bottom - rClient.top) < (nHeight * GetOption(scale)))
		{
			rect.bottom += GetSystemMetrics(SM_CYMENUSIZE);
			SetWindowPos(hwnd, HWND_NOTOPMOST, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_NOMOVE);
		}
	}
*/
}

bool init_display_depth ()
{
/*
	// Get the current display depth
	HDC hdc = GetDC(NULL);
	UINT uBPP = GetDeviceCaps(hdc, PLANES) * GetDeviceCaps(hdc, BITSPIXEL);
	ReleaseDC(NULL, hdc);
*/

	UINT uBPP = 16;

	// If a display already exists, delete the object
	if (g_pDisplay)
		delete g_pDisplay;

	BYTE* pbFrame;
	LONG lPitch;
	if (Frame::Lock(&pbFrame, &lPitch))
	{
		Frame::Unlock();

		// Create the new display
		if (GetOption(sourcesize) == 1)
		{
			switch (uBPP)
			{
				case 8:		g_pDisplay = new CDisplay1x1<BYTE>(pbFrame,lPitch);		break;
				case 16:	g_pDisplay = new CDisplay1x1<WORD>(pbFrame,lPitch);		break;
//				case 24:	g_pDisplay = new CDisplay1x1_24<BYTE>(pbFrame,lPitch);	break;
				case 32:	g_pDisplay = new CDisplay1x1<DWORD>(pbFrame,lPitch);	break;
			}
		}
		else
		{
			switch (uBPP)
			{
				case 8:		g_pDisplay = new CDisplay2x1<BYTE>(pbFrame,lPitch);		break;
				case 16:	g_pDisplay = new CDisplay2x1<WORD>(pbFrame,lPitch);		break;
//				case 24:	g_pDisplay = new CDisplay2x1_24<BYTE>(pbFrame,lPitch);	break;
				case 32:	g_pDisplay = new CDisplay2x1<DWORD>(pbFrame,lPitch);	break;
			}
		}

		if (g_pDisplay)
		{
			// Restore the original display mode
			g_pDisplay->SetMode((VMPR_SCREEN_MODE >> 5) + 1);
			return true;
		}

		Message(msgError, "Sorry, %d-bit mode is not supported!", uBPP);
	}

	return false;
}
