//	Notes:
//		Currently only supports MIDI OUT using the default MIDI OUT device
//
//		Will have a device selection sub-menu soon, as with the joystick
//		selection done by the Input module.

#include "stdafx.h"

#include "Util.h"


namespace MIDI
{
HMIDIOUT	hMidiOut;		// Handle for Windows MIDI OUT device

BYTE		abOut[256];		// Buffer to build up MIDI OUT messages
int			nOut;			// Number of bytes currently in abOut


bool Init ()
{
	// No MIDI message data yet
	ZeroMemory(abOut, sizeof abOut);
	nOut = 0;

	// For now just open the default MIDI OUT device
	midiOutOpen(&hMidiOut, MIDI_MAPPER, 0, 0, CALLBACK_NULL);
	return true;
}

void Exit ()
{
	if (hMidiOut)
		midiOutClose(hMidiOut);
}


BYTE In ()
{
	// No MIDI IN support yet
	return 0x00;
}

void Out (BYTE bVal_)
{
	// Protect against very long System Exclusive blocks
	if ((nOut == (sizeof abOut - 1)) && bVal_ != 0xf7)
	{
		TRACE("!!! MIDI: System Exclusive buffer overflow, discarding %#02x\n", bVal_);
		return;
	}

	// Do we have the start of a message while an incomplete message remains?
	if (nOut && (bVal_ & 0x80))
	{
		TRACE("!!! MIDI: Discarding incomplete %d byte message\n", nOut);
		nOut = 0;
	}

	// Is the start of the message a non-status byte?
	else if (!nOut && !(bVal_ & 0x80))
	{
		// Use the previous status byte if there is one
		if (abOut[0] & 0x80)
			nOut = 1;

		// Discard the byte as there isn't much we can do with it
		else
		{
			TRACE("!!! MIDI: Discarding leading non-status byte: %#02x\n", bVal_);
			return;
		}
	}

	// Add the new byte to the message we're building up
	abOut[nOut++] = bVal_;

	// Spot the end of a System Exclusive variable length block (we don't do anything with it yet)
	if (abOut[0] == 0xf0 && bVal_ == 0xf7)
		TRACE("MIDI: Variable block of %d bytes\n", nOut - 2);

	// Break out if the command we're building up hasn't got the required number of parameters yet
	else if (((abOut[0] & 0xfd) == 0xf1) || ((abOut[0] & 0xe0) == 0xc0))	// 1 byte
	{
		if (nOut != 2)
			return;
	}
	else if ((abOut[0] & 0xf0) == 0xf0)	// 0 bytes
	{
		if (nOut != 1)
			return;
	}
	else
	{
		if (nOut != 3)
			return;
	}

#ifdef _DEBUG
	switch (nOut)
	{
		case 1:		TRACE("MIDI: Sending 1 byte message from: %02x\n", abOut[0]);															break;
		case 2:		TRACE("MIDI: Sending 2 byte message from: %02x %02x\n", abOut[0], abOut[1]);											break;
		case 3:		TRACE("MIDI: Sending 3 byte message from: %02x %02x %02x\n", abOut[0], abOut[1], abOut[2]);								break;
		case 4:		TRACE("MIDI: Sending 4 byte message from: %02x %02x %02x %02x\n", abOut[0], abOut[1], abOut[2], abOut[3]);				break;
		default:	TRACE("MIDI: Sending %d byte message from: %02x %02x %02x %02x ...\n", nOut, abOut[0], abOut[1], abOut[2], abOut[3]);	break;
	}
#endif

	// Output the MIDI message
	midiOutShortMsg(hMidiOut, *reinterpret_cast<DWORD*>(&abOut));

	// Prepare for the next message, clearing out 
	nOut = abOut[1] = abOut[2] = abOut[3] = 0;
}


};	// namespace MIDI
